(function (window, document, $, Drupal) {
  var settings = function() {
    return Drupal.settings.webformPublicationsAjaxImprover || {};
  };

  var isAjaxHandlerReadyForAlteration = function () {
    return (
      ('ajax' in Drupal)
      &&
      (settings().inputId in Drupal.ajax)
      &&
      ('options' in Drupal.ajax[settings().inputId])
      &&
      ('beforeSubmit' in Drupal.ajax[settings().inputId].options)
      &&
      ('function' == typeof Drupal.ajax[settings().inputId].options.beforeSubmit)
      &&
      (ourBeforeSubmitHandler != Drupal.ajax[settings().inputId].options.beforeSubmit)
    );
  };

  var isValid = function (userInput) {
    return (new RegExp(settings().regExp)).test(userInput);
  };

  var previousValue = '';

  var hasChanged = function (userInput) {
    var isChanged = (previousValue != userInput);
    previousValue = userInput;
    return isChanged;
  };

  var oldBeforeSubmitHandler = function (form_values, element, options) {
    // Dummy placeholder until this var holds the real previous handler
    // function. See alterAjaxHandler()
    return true;
  };

  var ourBeforeSubmitHandler = function (form_values, element, options) {
    for (var i = 0; i < form_values.length; i++) {
      if (
        (settings().inputName == form_values[i].name)
        &&
        isValid(form_values[i].value)
        &&
        hasChanged(form_values[i].value)
      ) {
        return oldBeforeSubmitHandler(form_values, element, options);
      }
    }
    return false;
  };

  var alterAjaxHandler = function () {
    oldBeforeSubmitHandler = Drupal.ajax[settings().inputId].options.beforeSubmit;
    Drupal.ajax[settings().inputId].options.beforeSubmit = ourBeforeSubmitHandler;
  }

  Drupal.behaviors.webformPublicationsAjaxImprover = {
    attach: function (context, drupalSettings) {
      if (isAjaxHandlerReadyForAlteration()) {
        alterAjaxHandler();
      }
      else {
        $('#' + settings().inputId).once('webformPublicationsAjaxImprover').focus(function () {
          if (isAjaxHandlerReadyForAlteration()) {
            alterAjaxHandler();
          }
        });
      }
    }
  };

})(window, document, jQuery, Drupal);
