<?php
/**
 * @file
 * admin functions for webform_publications
 */

/**
 * Form constructor for publications overview table
 */
function webform_publications_filter_form($form, &$form_state, $country = '', $bounces = 0, $title_search = '', $regional = 0) {
  $form['#attributes'] = array('class' => array('search-form'));
  $form['basic'] = array('#type' => 'fieldset',
    '#title' => t('Filter Publications'),
    '#attributes' => array('class' => array('container-inline')),
  );
  $options = webform_publications_countries();
  $options['all'] = ' - All - ';
  asort($options);
  $form['basic']['country'] = array(
    '#type' => 'select',
    '#options' => $options,
    '#title' => 'Country',
    '#default_value' => $country,
  );
  $form['basic']['regional'] = array(
    '#type' => 'checkbox',
    '#title' => 'Only regional publications',
    '#default_value' => $regional,
  );
  $form['basic']['title_search'] = array(
    '#type' => 'textfield',
    '#title' => 'Title Search',
    '#default_value' => $title_search,
  );
  $form['basic']['bounces'] = array(
    '#type' => 'checkbox',
    '#title' => 'Only publications with bounces',
    '#default_value' => $bounces,
  );
  $form['basic']['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Filter'),
    '#submit' => array('webform_publications_filter_form_submit'),
  );
  return $form;
}

/**
 * Submit handler for country filter
 */
function webform_publications_filter_form_submit($form, &$form_state) {
  $form_state['redirect'] = 'admin/config/content/webform_publications/' . trim($form_state['values']['country'] . '/' . $form_state['values']['bounces'] . '/' . $form_state['values']['title_search'] . '/' . $form_state['values']['regional']);
}

/**
 * Page callback: Webform publications settings
 *
 * @see webform_publications_menu()
 */
function webform_publications_admin($country = NULL, $bounces = NULL, $title_search = NULL, $regional = NULL) {
  // Add the filter form above the table
  $build['form'] = drupal_get_form('webform_publications_filter_form', $country, $bounces, $title_search, $regional);

  // Setup header for table presentation.
  $header = array(
    array('data' => t('Name'), 'field' => 'title', 'sort' => 'asc'),
    array('data' => t('Country'), 'field' => 'country_code'),
    array('data' => t('Address'), 'field' => 'address'),
    array('data' => t('Bounces'), 'field' => 'bounces'),
    array('data' => t('Regional'), 'field' => 'state_wide'),
    array('data' => t('Operations'), 'colspan' => '2')
  );

  // Fetch all table records.
  $query = db_select('webform_publications', 'wp')->extend('PagerDefault')->extend('TableSort');
  if ($country && $country != 'all') {
    $query->condition('country_code', $country);
  }

  if (!empty($title_search)) {
    $query->condition('title', '%' . db_like($title_search) . '%', 'LIKE');
  }

  if ($regional) {
    $query->condition('statewide', '1');
  }

  $query->leftJoin('webform_publications_bounce', 'wpb', 'wp.newsid = wpb.newsid');
  $query->addExpression('COUNT(wpb.newsid)', 'bounces');
  $query->groupBy('wp.title', 'wp.country_code', 'wp.address', 'wp.phone');

  if ($bounces) {
    $query->havingCondition('bounces', 0, '>');
  }

  $result = $query
    ->fields('wp')
    ->OrderByHeader($header)
    ->limit(50)
    ->execute();

  // Dump results into array of formatted rows.
  $rows = array();
  $destination = drupal_get_destination();
  foreach ($result as $paper) {

    $statewide = ($paper->statewide) ? ' - statewide' : '';
    $regional = $paper->statewide ? t('Yes') : t('No');
    $country = isset($paper->country_code) ? check_plain($paper->country_code) : '';
    $title = ($paper->homepage != NULL ? l($paper->title, $paper->homepage) : $paper->title);
    $address = check_plain($paper->address)
      . '<br /> '. check_plain($paper->city)
      . ', ' . check_plain($paper->state)
      . ' ' . check_plain($paper->zip);

    $operations = array(
      'edit' => array(
        'title' => t('edit'),
        'href' => 'admin/config/content/webform_publications/edit/'. $paper->newsid,
        'query' => $destination,
      ),
      'delete' => array(
        'title' => t('delete'),
        'href' => 'admin/config/content/webform_publications/delete/'. $paper->newsid . '/' . $paper->title,
        'query' => $destination,
      ),
    );


    $rows[] = array(
      'data' => array(
        'name' => $title . $statewide . '<br />' . l($paper->email, 'mailto:' . $paper->email),
        'country' => $country,
        'address' => $address,
        'bounces' => !$bounces ? $bounces : l($paper->bounces, 'admin/config/content/webform_publications/bounces/' . $paper->newsid),
        'state_wide' => $regional,
        'operations' => array(
          'data' => array(
            '#theme' => 'links',
            '#links' => $operations,
            '#attributes' => array('class' => array('links', 'inline', 'nowrap')),
          ),
        ),
      ),
    );

  }

  // Return themed paper data in table.
  $build['webform_publications_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('No editors available.'),
  );
  $build['webform_publications_pager'] = array('#theme' => 'pager');
  return $build;
}

function webform_publications_admin_edit_form($form_id, $form_state, $newsid = NULL) {

  // Set default values to NULL  if $newsid = NULL.
  If ($newsid == NULL) {
    $publication = array(
      'newsid' => NULL,
      'title' => NULL,
      'address' => NULL,
      'city' => NULL,
      'state' => NULL,
      'zip' => NULL,
      'phone' => NULL,
      'fax' => NULL,
      'email' => NULL,
      'homepage' => NULL,
      'statewide' => NULL,
      'country_code' => NULL,
    );
  }

  // Query DB for publication based on newsid.
  $result = db_query('SELECT * FROM {webform_publications} WHERE newsid = :newsid ORDER BY title ASC', array(':newsid' => $newsid));

  // Count results.
  $count = count($result);
  // If one result, populate array with values from DB.
  if($count == 1) {
    foreach ($result as $row) {
       $publication = array(
        'newsid' => $row->newsid,
        'title' => $row->title,
        'address' => $row->address,
        'city' => $row->city,
        'state' => $row->state,
        'zip' => $row->zip,
        'phone' => $row->phone,
        'fax' => $row->fax,
        'email' => $row->email,
        'homepage' => $row->homepage,
        'statewide' => $row->statewide,
        'country_code' => $row->country_code,
      );
    }
  }
  else {
    // Throw error if no records exist for newsid.
    if ($count < 1) {
      drupal_set_message(t('The requested publication does not exist in the database.'), 'error');
    }
    // Throw error if multiple records exist with same newsid.
    if ($count > 1) {
      drupal_set_message(t('Multiple publications exist in the database with the same unique ID.'), 'error');
    }
  }

  // Populate form with values from array.
  $form['title'] = array(
    '#type' => 'textfield',
    '#title' => t('Title'),
    '#default_value' => $publication['title'],
    '#required' => TRUE,
  );
  $form['address'] = array(
    '#type' => 'textfield',
    '#title' => t('Address'),
    '#default_value' => $publication['address'],
  );
  $form['city'] = array(
    '#type' => 'textfield',
    '#title' => t('City'),
    '#default_value' => $publication['city'],
  );
  $form['state'] = array(
    '#type' => 'textfield',
    '#title' => t('Province or State'),
    '#description' => t('Use abbreviation up to eight letters.'),
    '#default_value' => $publication['state'],
    '#size' => 8,
    '#maxlength' => 8,
  );
  $form['zip'] = array(
    '#type' => 'textfield',
    '#title' => t('Postal or zip code'),
    '#default_value' => $publication['zip'],
    '#required' => TRUE,
    '#maxlength' => 10,
  );
  $form['phone'] = array(
    '#type' => 'textfield',
    '#title' => t('Phone'),
    '#default_value' => $publication['phone'],
    '#maxlength' => 15,
  );
  $form['fax'] = array(
    '#type' => 'textfield',
    '#title' => t('Fax'),
    '#default_value' => $publication['fax'],
    '#maxlength' => 15,
  );
  $form['email'] = array(
    '#type' => 'textfield',
    '#title' => t('E-mail'),
    '#default_value' => $publication['email'],
    '#required' => TRUE,
  );
  $form['homepage'] = array(
    '#type' => 'textfield',
    '#title' => t('Homepage'),
    '#default_value' => $publication['homepage'],
  );
  $form['statewide'] = array(
    '#type' => 'checkbox',
    '#title' => t('Regional'),
    '#description' => t('Scope of publication is regional and not restricted to a single city.'),
    '#default_value' => $publication['statewide'],
  );
  $form['country_code'] = array(
    '#type' => 'textfield',
    '#title' => t('Country Code'),
    '#description' => t('Country code of publication.'),
    '#size' => 2,
    '#maxlength' => 2,
    '#default_value' => $publication['country_code'],
    '#required' => TRUE,
  );
  $form['#redirect'] = 'admin/config/content/webform_publications';
  $form['newsid'] = array('#type' => 'value', '#value' => $publication['newsid']);
  $form['submit'] = array('#type' => 'submit', '#value' => t('Submit'));

  return $form;
}

function webform_publications_admin_edit_form_validate($form, &$form_state) {
  //check to ensure email address is valid.
  if (!valid_email_address($form_state['values']['email'])) {
    form_set_error('email', t('The email address appears to be invalid.'));
  }
  // Validate country code.
  if (!ctype_alpha($form_state['values']['country_code'])) {
    form_set_error('country_code', t('The country code can only contain letters.'));
  }
}

function webform_publications_admin_edit_form_submit($form, &$form_state) {
  if ($form_state['values']['newsid']) {
    $num_updated = db_update('webform_publications')
      ->fields(array(
        'title' => $form_state['values']['title'],
        'address' => $form_state['values']['address'],
        'city' => $form_state['values']['city'],
        'state' => $form_state['values']['state'],
        'zip' => $form_state['values']['zip'],
        'phone' => $form_state['values']['phone'],
        'fax' => $form_state['values']['fax'],
        'email' => $form_state['values']['email'],
        'homepage' => $form_state['values']['homepage'],
        'statewide' => $form_state['values']['statewide'],
        'country_code' => $form_state['values']['country_code'],
      ))
      ->condition('newsid', $form_state['values']['newsid'])
      ->execute();
  }
  else {
    $newsid = db_insert('webform_publications')
      ->fields(array(
        'title' => $form_state['values']['title'],
        'address' => $form_state['values']['address'],
        'city' => $form_state['values']['city'],
        'state' => $form_state['values']['state'],
        'zip' => $form_state['values']['zip'],
        'phone' => $form_state['values']['phone'],
        'fax' => $form_state['values']['fax'],
        'email' => $form_state['values']['email'],
        'homepage' => $form_state['values']['homepage'],
        'statewide' => $form_state['values']['statewide'],
        'country_code' => $form_state['values']['country_code'],
      ))
      ->execute();
  }
  // Add the zipcode to the zipcodes table
  $location = array(
    'country' => $form_state['values']['country_code'],
    'postal_code' => $form_state['values']['zip'],
  );
  location_latlon_rough($location);
  $form_state['redirect'] = 'admin/config/content/webform_publications';
}

function webform_publications_admin_delete_form($form_id, $form_state, $newsid = NULL, $title = NULL) {

  $question = t('Are you sure you want to delete the publication %item?', array('%item' => $title));
  $path = 'admin/config/content/webform_publications/delete';
  $description = t('This action cannot be undone.');

  $form['newsid'] = array('#type' => 'hidden', '#value' => $newsid);
  $form['name'] = array('#type' => 'hidden', '#value' => $title);

  return confirm_form($form, $question, $path, $description, t('Delete'), t('Cancel'));
}

function webform_publications_admin_delete_form_submit($form, &$form_state) {
  $num_deleted = db_delete('webform_publications')
    ->condition('newsid', $form_state['values']['newsid'])
    ->execute();

  $t_args = array('%name' => $form_state['values']['name']);
  drupal_set_message(t('The publication %name has been deleted.', $t_args));
  watchdog('menu', 'Deleted publication %name.', $t_args, WATCHDOG_NOTICE);
  $form_state['redirect'] = 'admin/config/content/webform_publications';
}

/**
 * Page callback: Webform publications bounces
 *
 * @see webform_publications_menu()
 */
function webform_publications_bounces($newsid) {
  // Setup header for table presentation.
  $header = array(
    array('data' => t('Date'), 'field' => 'created'),
    array('data' => t('Reason'), 'field' => 'reason', 'sort' => 'asc'),
    array('data' => t('Status'), 'field' => 'status'),
    array('data' => t('Operations'), 'colspan' => '2')
  );

  // Fetch all table records.
  $query = db_select('webform_publications_bounce', 'wpb')->extend('PagerDefault')->extend('TableSort');

  $result = $query
    ->fields('wpb')
    ->condition('newsid', $newsid)
    ->OrderByHeader($header)
    ->limit(50)
    ->execute();

  // Dump results into array of formatted rows.
  $rows = array();
  $destination = drupal_get_destination();
  foreach ($result as $bounce) {

    $operations = array(
      'delete' => array(
        'title' => t('delete'),
        'href' => 'admin/config/content/webform_publications/bounces/delete/'. $newsid . '/' . $bounce->created,
        'query' => $destination,
      ),
    );

    $rows[] = array(
      'data' => array(
        'date' => format_date($bounce->created),
        'reason' => $bounce->reason,
        'status' => $bounce->status,
        'operations' => array(
          'data' => array(
            '#theme' => 'links',
            '#links' => $operations,
            '#attributes' => array('class' => array('links', 'inline', 'nowrap')),
          ),
        ),
      ),
    );

  }

  // Return themed paper data in table.
  $build['webform_publications_bounce_table'] = array(
    '#theme' => 'table',
    '#header' => $header,
    '#rows' => $rows,
    '#empty' => t('No editors available.'),
  );
  $build['webform_publications_pager'] = array('#theme' => 'pager');
  return $build;
}

function webform_publications_bounces_delete_form($form_id, $form_state, $newsid = NULL, $created = NULL) {

  $question = t('Are you sure you want to delete this bounce?');
  $path = 'admin/config/content/webform_publications/bounces/delete';
  $description = t('This action cannot be undone.');

  $form['newsid'] = array('#type' => 'hidden', '#value' => $newsid);
  $form['created'] = array('#type' => 'hidden', '#value' => $created);

  return confirm_form($form, $question, $path, $description, t('Delete'), t('Cancel'));
}

function webform_publications_bounces_delete_form_submit($form, &$form_state) {
  $num_deleted = db_delete('webform_publications_bounce')
    ->condition('newsid', $form_state['values']['newsid'])
    ->condition('created', $form_state['values']['created'])
    ->execute();

  drupal_set_message(t('The bounce has been deleted.'));
  $form_state['redirect'] = 'admin/config/content/webform_publications/bounces/' . $form_state['values']['newsid'];
}

